import inquirer from 'inquirer';
import inquirerFileTreeSelection from 'inquirer-file-tree-selection-prompt';
import { lstatSync } from 'node:fs';
import { extname } from 'node:path';
import showProblemDescription from './description.js';
import getSolveData from '../utils/getSolveData.js';
import postSolveData, { createSubmitData } from '../utils/postSolveData.js';
import { printInfo, printError, printSuccess, printTip, printValue } from '../utils/messages.js';
import { contestSubmit, pageData } from '../lib/routes.js';
import { sendSolutionOption, descriptionOption, quitOption } from '../lib/options.js';
import { handleSubmitStatus } from '../utils/printTable.js';
export const send = async (SessionId, route, sendData, resUrl) => {
    postSolveData(SessionId, route, sendData)
        .then((res) => {
        const responseUrl = res.request.res.responseUrl;
        if (responseUrl === resUrl) {
            printSuccess('File has been successfully sent!');
        }
        else {
            printError('Error while sending the file');
        }
    })
        .catch((error) => {
        printError(error.message);
    });
};
const sendContestSolution = async (SessionId, problemShortName, id, filePath) => {
    const contestSendData = await createSubmitData(SessionId, id, problemShortName, filePath);
    const resUrl = `https://solve.edu.pl/contests/view/${id}`;
    send(SessionId, contestSubmit + id, contestSendData, resUrl);
};
const showProblems = async (SessionId, contestId, problemId, filePath) => {
    const { problems, own_results } = await getSolveData(SessionId, pageData, contestId);
    if (problemId) {
        const problem = problems.find(({ short_name, id }) => id === problemId || short_name.toLowerCase() === problemId.toLowerCase());
        if (problem === undefined) {
            printError('No problems found');
            return;
        }
        const { short_name: problemShortName, name: problemName } = problem;
        printValue('Name', problemName);
        printValue('Short name', problemShortName);
        if (!filePath) {
            filePath = await selectFile();
        }
        if (!filePath) {
            printError('File path was not selected!');
            return;
        }
        sendContestSolution(SessionId, problemShortName, contestId, filePath);
    }
    else {
        const choices = problems.map(({ short_name, name, id, contest_id, problem_id, multiplier }) => {
            return {
                name: `${short_name} - ${handleSubmitStatus(own_results[Number(id)].solution_status)} - ${name}`,
                value: {
                    name,
                    id,
                    contest_id,
                    problem_id,
                    short_name,
                    multiplier,
                },
            };
        });
        await inquirer
            .prompt([
            {
                type: 'list',
                name: 'problem',
                message: 'Select problem',
                choices: choices,
            },
        ])
            .then(async ({ problem }) => {
            showProblemInfo(problem);
            await showProblemOptions(SessionId, contestId, problem.short_name, problem.id);
        });
    }
};
const showProblemOptions = async (SessionId, contestId, problemShortName, id) => {
    const choices = [sendSolutionOption, descriptionOption, quitOption];
    await inquirer
        .prompt([
        {
            type: 'list',
            name: 'option',
            message: 'Select option',
            choices: choices,
        },
    ])
        .then(async ({ option }) => {
        switch (option) {
            case sendSolutionOption: {
                const filePath = await selectFile();
                if (filePath) {
                    await sendContestSolution(SessionId, problemShortName, contestId, filePath);
                }
                break;
            }
            case descriptionOption:
                await showProblemDescription(id);
                break;
        }
    });
};
const showProblemInfo = ({ name, id, contest_id, problem_id, short_name }) => {
    printTip('Problem Info');
    printInfo('Name', name);
    printInfo('ID', id);
    printInfo('Contest ID', contest_id);
    printInfo('Problem ID', problem_id);
    printInfo('Short name', short_name);
    printInfo('Description link', `https://solve.edu.pl/contests/download_desc/${id}`);
};
const isCppFile = (value, ext = '.cpp') => {
    if (extname(value) === ext || extname(value) === '') {
        return true;
    }
};
export const selectFile = async (ext) => {
    inquirer.registerPrompt('file-tree-selection', inquirerFileTreeSelection);
    return await inquirer
        .prompt([
        {
            type: 'file-tree-selection',
            name: 'filePath',
            message: 'Select file',
            onlyShowValid: true,
            validate: (val) => isCppFile(val, ext),
        },
    ])
        .then(async ({ filePath }) => {
        if (!lstatSync(filePath).isDirectory()) {
            return filePath;
        }
        else {
            printError("You can't select a directory");
            printTip('Press <tab> to expand the directory and select a valid file');
            return;
        }
    });
};
export default showProblems;
//# sourceMappingURL=send.js.map